/*********************************************************************
* lescegra                                                           *
*                                                                    *
* http://geeky.kicks-ass.org/projects/lescegra.html                  *
*                                                                    *
* Copyright 2003-2004 by Enno Cramer <uebergeek@web.de>              *
*                                                                    *
* This library is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU Library General Public        *
* License as published by the Free Software Foundation; either       *
* version 2 of the License, or (at your option) any later version.   *
*                                                                    *
* This library is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
* Library General Public License for more details.                   *
*                                                                    *
* You should have received a copy of the GNU Library General Public  *
* License along with this library; if not, write to the Free         *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. *
*********************************************************************/

#ifndef LSG_FRUSTUM_H
#define LSG_FRUSTUM_H 1

/**
 * \file  frustum.h
 * \brief View Frustum
 */

#include <lescegra/base/object.h>

#include <lescegra/base/vertex.h>
#include <lescegra/base/matrix.h>

/**
 * \brief Clipping plane
 *
 * A view frustum clipping plane
 */
typedef struct {
    Vertex normal;
    float distance;
} LsgPlane;

typedef struct LsgFrustum LsgFrustum;
typedef struct LsgFrustumClass LsgFrustumClass;

/**
 * \brief View Frustum
 *
 * A view frustum defined by two matrizes (modelview and projection)
 * and the six clipping planes.
 */
struct LsgFrustum {
    LsgObject parent;
    
    Matrix projection;
    Matrix modelview;
    LsgPlane planes[6];
};

struct LsgFrustumClass {
    LsgObjectClass parent;
};

LsgClassID LsgFrustum_classID(void);

#define IS_LSG_FRUSTUM(instance) \
    LSG_CLASS_INSTANCE_OF((instance), LsgFrustum_classID())

#define LSG_FRUSTUM(instance) \
    LSG_CLASS_INSTANCE_CAST(LsgFrustum*, LsgFrustum_classID(), (instance))

#define LSG_FRUSTUM_CLASS(class) \
    LSG_CLASS_CAST(LsgFrustumClass*, LsgFrustum_classID(), (class))

/**
 * \relates LsgFrustum
 * Allocate and initialize a view frustum.
 * @param projection    The projection matrix
 * @param modelview     The modelview matrix
 * @return The view frustum as defined by projection and modelview
 */
LsgFrustum* LsgFrustum_create(const Matrix projection, const Matrix modelview);

/**
 * \relates LsgFrustum
 * Constructor method for LsgFrustum.
 * @param self          The instance variable
 * @param projection    The projection matrix
 * @param modelview     The modelview matrix
 */
void LsgFrustum_init(LsgFrustum* self, const Matrix projection, const Matrix modelview);

/**
 * \relates LsgFrustum
 * Transform a view frustum.
 * @param self          The instance variable
 * @param projection    The transformation matrix for the projection matrix
 * @param modelview     The transformation matrix for the modelview matrix
 */
void LsgFrustum_transform(LsgFrustum* self, const Matrix projection, const Matrix modelview);

/**
 * \relates LsgFrustum
 * Update the clipping planes after transforming the defining projection
 * and/or modelview matrix.
 * @param self          The instance variable
 */
void LsgFrustum_updatePlanes(LsgFrustum* self);

#endif
