/*********************************************************************
* lescegra                                                           *
*                                                                    *
* http://geeky.kicks-ass.org/projects/lescegra.html                  *
*                                                                    *
* Copyright 2003-2004 by Enno Cramer <uebergeek@web.de>              *
*                                                                    *
* This library is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU Library General Public        *
* License as published by the Free Software Foundation; either       *
* version 2 of the License, or (at your option) any later version.   *
*                                                                    *
* This library is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
* Library General Public License for more details.                   *
*                                                                    *
* You should have received a copy of the GNU Library General Public  *
* License along with this library; if not, write to the Free         *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. *
*********************************************************************/

#ifndef LSG_MD2MODEL_H
#define LSG_MD2MODEL_H 1

/**
 * \file  md2model.h
 * \brief Quake II model container and loader
 */

#include <lescegra/geom/mesh.h>

#include <lescegra/base/vertex.h>

typedef struct LsgMD2Model LsgMD2Model;
typedef struct LsgMD2ModelClass LsgMD2ModelClass;

/**
 * \ingroup geometry
 * \brief   Quake II model container
 *
 * Container class for a Quake II model with animation frames and skin
 * textures.
 */
struct LsgMD2Model {
    LsgMesh parent;

    LsgTriangle* triangles;
    LsgTexCoord* tex_coords;
    Vertex* normals;
    Vertex* vertices;

    LsgOctree** octrees;

    int frame_count;
    int vertex_count;

    int frame;
};

struct LsgMD2ModelClass {
    LsgMeshClass parent;
};

LsgClassID LsgMD2Model_classID(void);

#define IS_LSG_MD2MODEL(instance) \
    LSG_CLASS_INSTANCE_OF((instance), LsgMD2Model_classID())

#define LSG_MD2MODEL(instance) \
    LSG_CLASS_INSTANCE_CAST(LsgMD2Model*, LsgMD2Model_classID(), (instance))

#define LSG_MD2MODEL_CLASS(class) \
    LSG_CLASS_CAST(LsgMD2ModelClass*, LsgMD2Model_classID(), (class))

/**
 * Allocate and initialize a LsgMD2Model instance.
 * @param filename  The name of the file containing the Quake II model data
 * @return A new LsgMD2Model instance
 */
LsgMD2Model* LsgMD2Model_create(const char* filename);

/**
 * Constructor method for LsgMD2Model. Load a Quake II model from the given
 * file.
 * @param self      The instance variable
 * @param filename  The name of the Quake II model file
 */
int  LsgMD2Model_init(LsgMD2Model* self, const char* filename);

void LsgMD2Model_frame(LsgMD2Model* self, int frame);

#endif
