/*********************************************************************
* lescegra                                                           *
*                                                                    *
* http://geeky.kicks-ass.org/projects/lescegra.html                  *
*                                                                    *
* Copyright 2003-2004 by Enno Cramer <uebergeek@web.de>              *
*                                                                    *
* This library is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU Library General Public        *
* License as published by the Free Software Foundation; either       *
* version 2 of the License, or (at your option) any later version.   *
*                                                                    *
* This library is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
* Library General Public License for more details.                   *
*                                                                    *
* You should have received a copy of the GNU Library General Public  *
* License along with this library; if not, write to the Free         *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. *
*********************************************************************/

#ifndef LSG_VLTME_H
#define LSG_VLTME_H 1

#include <lescegra/sg/node.h>

#include <lescegra/util/image.h>

typedef struct {
    Vertex n;
    Vertex v;
    float error;
    float radius;
} LsgVLTME_Node;

typedef struct {
    unsigned int* indices;
    unsigned int size;
    unsigned int count;
    unsigned int parity;
} LsgVLTME_Mesh;

typedef struct {
    const LsgFrustum* frustum;
    Vertex*           eye;
    float             error_threshold;
} LsgVLTME_MeshParams;

typedef struct LsgVLTME LsgVLTME;
typedef struct LsgVLTMEClass LsgVLTMEClass;

struct LsgVLTME {
    LsgNode parent;

    const LsgVLTME_Node* nodes;
    unsigned int levels;

    LsgVLTME_Mesh* mesh;

    Vertex* eye;
    float error_threshold;
};

struct LsgVLTMEClass {
    LsgNodeClass parent;
};

LsgClassID LsgVLTME_classID(void);

#define IS_LSG_VLTME(instance) \
    LSG_CLASS_INSTANCE_OF((instance), LsgVLTME_classID())

#define LSG_VLTME(instance) \
    LSG_CLASS_INSTANCE_CAST(LsgVLTME*, LsgVLTME_classID(), (instance))

#define LSG_VLTME_CLASS(class) \
    LSG_CLASS_CAST(LsgVLTMEClass*, LsgVLTME_classID(), (class))

LsgVLTME* LsgVLTME_create(
    const LsgVLTME_Node* nodes,
    unsigned int levels
);

LsgVLTME* LsgVLTME_createFromImage(
    const LsgImage* img,
    const Matrix transform
);

void LsgVLTME_init(
    LsgVLTME* self,
    const LsgVLTME_Node* nodes,
    unsigned int levels
);

LsgVLTME_Mesh* LsgVLTME_createMesh(void);

void LsgVLTME_freeMesh(LsgVLTME_Mesh* mesh);

void LsgVLTME_buildMesh(
    LsgVLTME_Mesh* mesh,
    const LsgVLTME_MeshParams* params,
    const LsgVLTME_Node* nodes,
    unsigned int levels
);

#endif
