/*********************************************************************
* lescegra                                                           *
*                                                                    *
* http://geeky.kicks-ass.org/projects/lescegra.html                  *
*                                                                    *
* Copyright 2003-2004 by Enno Cramer <uebergeek@web.de>              *
*                                                                    *
* This library is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU Library General Public        *
* License as published by the Free Software Foundation; either       *
* version 2 of the License, or (at your option) any later version.   *
*                                                                    *
* This library is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
* Library General Public License for more details.                   *
*                                                                    *
* You should have received a copy of the GNU Library General Public  *
* License along with this library; if not, write to the Free         *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. *
*********************************************************************/

#ifndef LSG_MATERIAL_H
#define LSG_MATERIAL_H 1

/**
 * \file  material.h
 * \brief Material properties
 */

#include <lescegra/sg/group.h>

typedef struct LsgMaterial LsgMaterial;
typedef struct LsgMaterialClass LsgMaterialClass;

/**
 * \ingroup scene
 * \brief   Material properties
 *
 * Change OpenGL material parameters
 */
struct LsgMaterial {
    LsgGroup parent;

    LsgList* textures;

    float ambient[4];
    float diffuse[4];
    float specular[4];
    float shininess;
};

struct LsgMaterialClass {
    LsgGroupClass parent;
};

LsgClassID LsgMaterial_classID(void);

#define IS_LSG_MATERIAL(instance) \
    LSG_CLASS_INSTANCE_OF((instance), LsgMaterial_classID())

#define LSG_MATERIAL(instance) \
    LSG_CLASS_INSTANCE_CAST(LsgMaterial*, LsgMaterial_classID(), (instance))

#define LSG_MATERIAL_CLASS(class) \
    LSG_CLASS_CAST(LsgMaterialClass*, LsgMaterial_classID(), (class))

/**
 * \relates LsgMaterial
 * Allocate and initialize a material node.
 * @return A new material node
 */
LsgMaterial* LsgMaterial_create(void);

/**
 * \relates LsgMaterial
 * Constructor method for LsgMaterial. Create a new material node with default
 * color values.
 * @param self      The instance variable
 */
void LsgMaterial_init(LsgMaterial* self);

#endif
