#include <lescegra/base/endian.h>

#include <lescegra/base/error.h>

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include <config.h>

#ifdef WORDS_BIGENDIAN
# define HOST_ENDIANESS LSG_ENDIAN_BIG
#else
# define HOST_ENDIANESS LSG_ENDIAN_LITTLE
#endif

LsgEndian_Endianess LsgEndian_endianess(void) {
    static union {
        int  as_int;
        char as_char;
    } one = { 1 };
    
    if (one.as_char == 1) {
        return LSG_ENDIAN_LITTLE;
    } else {
        return LSG_ENDIAN_BIG;
    }
}

void LsgEndian_swap(void* buffer, int size) {
    char tmp;
    char* src;
    char* dst;
    
    src = (char*)buffer;
    dst = (char*)buffer + size - 1;
    while (src < dst) {
        tmp = *src;
        *src = *dst;
        *dst = tmp;
        ++src; --dst;
    }
}

void LsgEndian_swapArray(void* buffer, int size, int count) {
    int c;
    
    for (c = 0; c < count; ++c) {
        LsgEndian_swap((char*)buffer + c * size, size);
    }
}

void LsgEndian_swapShort(short int* buffer) {
    LsgEndian_swap(buffer, sizeof(short int));
}

void LsgEndian_swapInt(int* buffer) {
    LsgEndian_swap(buffer, sizeof(int));
}

void LsgEndian_swapLong(long int* buffer) {
    LsgEndian_swap(buffer, sizeof(long int));
}

int LsgEndian_readf(FILE* stream, const char* format, ...)
{
    va_list args;
    const char* c = format;
    int cont = 1, items = 0;

    va_start(args, format);

    while (cont && *c) {
        void* buffer;
        int size, order, count;
        int read;

        while (isspace(*c))
            ++c;

        /* extract optional repetition count */
        if (isdigit(*c)) {
            count = 0;

            while (isdigit(*c)) {
                count *= 10;
                count += *c - '0';
                ++c;
            }

            while (isspace(*c))
                ++c;

        } else {
            count = 1;
        }

        assert(*c != 0);

        /* extract field size and byte order */
        switch (*c++) {
            case 'b':
                size  = 1;
                order = LSG_ENDIAN_LITTLE;
                break;

            case 'B':
                size  = 1;
                order = LSG_ENDIAN_BIG;
                break;

            case 'w':
                size  = 2;
                order = LSG_ENDIAN_LITTLE;
                break;
                
            case 'W':
                size  = 2;
                order = LSG_ENDIAN_BIG;
                break;

            case 'd':
                size  = 4;
                order = LSG_ENDIAN_LITTLE;
                break;

            case 'D':
                size  = 4;
                order = LSG_ENDIAN_BIG;
                break;

            case 'q':
                size  = 8;
                order = LSG_ENDIAN_LITTLE;
                break;

            case 'Q':
                size  = 8;
                order = LSG_ENDIAN_BIG;
                break;

            default:
                LsgError_abortFormat(
                    __FILE__, "LsgEndian_readf", __LINE__,
                    "invalid format string: %s", format
                );
                return -1;
        }
        
        buffer = va_arg(args, void*);
        items += read = fread(buffer, size, count, stream);

        if ((size > 1) && (order != HOST_ENDIANESS))
            LsgEndian_swapArray(buffer, size, count);

        cont = read == count;
    }

    va_end(args);

    return items;
}
